package de.fitko.fitconnect.core;

import com.google.inject.Inject;
import com.nimbusds.jose.jwk.RSAKey;
import de.fitko.fitconnect.api.domain.auth.OAuthToken;
import de.fitko.fitconnect.api.domain.model.metadata.Metadata;
import de.fitko.fitconnect.api.domain.model.metadata.attachment.Attachment;
import de.fitko.fitconnect.api.domain.model.metadata.attachment.signature.Hash__1;
import de.fitko.fitconnect.api.domain.model.metadata.attachment.signature.Type;
import de.fitko.fitconnect.api.domain.model.metadata.data.Data;
import de.fitko.fitconnect.api.domain.model.submission.SubmissionRequest;
import de.fitko.fitconnect.api.domain.model.submission.CreatedSubmission;
import de.fitko.fitconnect.api.domain.validation.ValidationResult;
import de.fitko.fitconnect.api.exceptions.internal.AuthenticationException;
import de.fitko.fitconnect.api.services.Sender;
import de.fitko.fitconnect.api.services.auth.OAuthService;
import de.fitko.fitconnect.api.services.crypto.CryptoService;
import de.fitko.fitconnect.api.services.metadata.MetadataService;
import de.fitko.fitconnect.api.services.validation.CertificateValidator;
import de.fitko.fitconnect.core.crypto.HashUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

public class SubmissionSender implements Sender {

    private static final Logger logger = LoggerFactory.getLogger(SubmissionSender.class);

    private final OAuthService authService;
    private final CertificateValidator certificateValidator;
    private final CryptoService cryptoService;
    private final MetadataService metadataService;

    private HashUtil hashUtil = new HashUtil();

    @Inject
    public SubmissionSender(final OAuthService authService,
                            final CryptoService encryptionService,
                            final CertificateValidator certificateValidator,
                            final MetadataService metadataService) {
        this.authService = authService;
        this.cryptoService = encryptionService;
        this.certificateValidator = certificateValidator;
        this.metadataService = metadataService;
    }


    @Override
    public ValidationResult validatePublicKey(RSAKey publicKey) {
        return certificateValidator.validatePublicKey(publicKey);
    }

    @Override
    public Optional<Data> encryptSubmissionData(RSAKey publicKey, Data data) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public Optional<Attachment> encryptAttachment(RSAKey publicKey, Attachment attachment) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public Optional<OAuthToken> retrieveOAuthToken(String clientId, String clientSecret, String... scope) {
        try {
            final OAuthToken token = authService.authenticate(clientId, clientSecret, scope);
            logger.debug("successfully retrieved token {}", token.getAccessToken());
            return Optional.of(token);
        } catch (AuthenticationException e) {
            logger.error("client could not be authenticated", e.getMessage());
            return Optional.empty();
        }
    }

    @Override
    public Optional<Metadata> createMetadata(Data data, List<Attachment> attachments) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public Optional<Metadata> createMetadata(List<Attachment> attachments) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public Optional<CreatedSubmission> createSubmission(SubmissionRequest submission) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public Optional<CreatedSubmission> sendSubmission(UUID submissionId, Metadata encryptedMetadata) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public void uploadAttachments(UUID submissionId, List<Attachment> encryptedAttachments) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public Optional<Attachment> createAttachment(File file) {
        try {
            String hashedFileContent = hashUtil.createHashFromFile(file);
            Hash__1 hash = new Hash__1();
            hash.setType(Type.SHA_512);
            hash.setContent(hashedFileContent);
            return Optional.ofNullable(Attachment.builder()
                    .attachmentId(UUID.randomUUID())
                    .filename(file.getName())
                    .hash(hash)
                    .build());
        } catch (IOException e) {
            logger.error("Attachment could not be read", e);
            return Optional.empty();
        }
    }

    @Override
    public RSAKey getEncryptionKeyForDestination(UUID destinationId) {
        throw new UnsupportedOperationException("not yet implemented");
    }

}
