package de.fitko.fitconnect.core.http;

import com.google.common.base.Strings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.web.client.RestTemplate;

import java.net.InetSocketAddress;
import java.net.Proxy;

public class ProxyConfig {

    private static final Logger logger = LoggerFactory.getLogger(ProxyConfig.class);

    private final String host;
    private final int port;

    public ProxyConfig(String host, int port) {
        this.host = host;
        this.port = port;
    }

    public RestTemplate proxyRestTemplate() {
        if (!hasProxySet()) {
            logger.info("no proxy configured");
            return new RestTemplate();
        }

        logger.info("using proxy {}", this);
        var requestFactory = new SimpleClientHttpRequestFactory();
        var proxy = new Proxy(Proxy.Type.HTTP, new InetSocketAddress(host, port));
        requestFactory.setProxy(proxy);
        return new ProxyRestTemplate(requestFactory);
    }

    public boolean hasProxySet() {
        return !Strings.isNullOrEmpty(host) && port > 0;
    }

    @Override
    public String toString() {
        return String.format("ProxyConfig {host='%s', port=%d}", host, port);
    }
}