package de.fitko.fitconnect.core.validation;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import de.fitko.fitconnect.api.domain.model.metadata.Metadata;
import de.fitko.fitconnect.api.services.validation.MetadataValidator;
import de.fitko.fitconnect.api.domain.validation.ValidationResult;
import de.fitko.fitconnect.core.crypto.HashUtil;
import org.everit.json.schema.Schema;
import org.everit.json.schema.ValidationException;
import org.everit.json.schema.loader.SchemaLoader;
import org.json.JSONObject;
import org.json.JSONTokener;

public class MetadataSubmissionValidator implements MetadataValidator {

    public static final String DEFAULT_SCHEMA_PATH = "schemas/metadata.schema.json";
    private static final HashUtil verifier = new HashUtil();
    private final ObjectMapper mapper = new ObjectMapper();

    @Override
    public ValidationResult validateMetadataSchema(Metadata metadata, String jsonSchema) {
        final JSONObject schemaObject = new JSONObject(new JSONTokener(jsonSchema));
        try {
            JSONObject jsonSubject = toJsonObject(metadata);
            Schema schema = SchemaLoader.load(schemaObject);
            schema.validate(jsonSubject);
        } catch (JsonProcessingException | ValidationException e) {
            return ValidationResult.error(e);
        }
        return ValidationResult.ok();
    }

    @Override
    public ValidationResult validateMetadataHashValues(Metadata metadata) {
        return ValidationResult.ok();
    }

    private JSONObject toJsonObject(Metadata metadata) throws JsonProcessingException {
        return new JSONObject(new JSONTokener(mapper.writeValueAsString(metadata)));
    }

    private String loadSchemaFile(final String jsonSchemaPath) {
        return "";
    }
}
