package fitconnect.impl.crypto;

import com.nimbusds.jose.JOSEException;
import com.nimbusds.jose.jwk.KeyUse;
import com.nimbusds.jose.jwk.RSAKey;
import com.nimbusds.jose.jwk.gen.RSAKeyGenerator;
import de.fitko.fitconnect.api.services.crypto.CryptoService;
import de.fitko.fitconnect.core.crypto.JWECryptoService;
import org.junit.jupiter.api.Test;

import java.util.UUID;

import static org.junit.jupiter.api.Assertions.*;

class JWECryptoServiceTest {

    final CryptoService underTest = new JWECryptoService();

    @Test
    void encryptData() throws Exception {

        // Given
        final RSAKey privateKey = generateRsaKey(4096);
        final RSAKey publicKey = privateKey.toPublicJWK();

        // When
        String encryptedData = underTest.encryptString(publicKey, "some foo");

        // Then
        assertNotNull(encryptedData);
        assertNotEquals("some foo", encryptedData);
    }

    @Test
    void decryptData() throws Exception {

        // Given
        final RSAKey privateKey = generateRsaKey(4096);
        final RSAKey publicKey = privateKey.toPublicJWK();
        String encryptedData = underTest.encryptString(publicKey, "some foo");

        // When
        String decrypted = underTest.decryptString(privateKey, encryptedData);

        // Then
        assertNotNull(decrypted);
        assertEquals("some foo", decrypted);
    }

    public static RSAKey generateRsaKey(int size) throws JOSEException {
        return new RSAKeyGenerator(size)
                .keyUse(KeyUse.ENCRYPTION)
                .keyID(UUID.randomUUID().toString())
                .generate();
    }

}